﻿namespace HeuristicLab.BenchmarkSuite.Problems {
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Char;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Integer;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.String;

  public class Syllables : BenchmarkSuiteDataDescriptor {
    private const string name = "Syllables";
    private const string fileName = "Syllables.csv";
    private const string description = "Given a string containing symbols, spaces, digits, and lowercase letters, count the number of occurrences of vowels(a, e, i, o, u, y) in the string and print that number as X in The number of syllables is X.";

    protected override string FileName { get { return fileName; } }
    public override string Name { get { return name; } }
    public override string Description { get { return description; } }
    protected override int InputArgumentCount { get { return 1; } }
    protected override int OutputArgumentCount { get { return 1; } }

    public override ProblemData CreateProblemData() {
      return new ProblemData {
        Name = Name,
        Description = Description,
        Examples = CloneExamples(),
        BestResult = 0,
        WorstResult = 100,
        InputArgumentTypes = new[] { ExampleArgumentType.String },
        OutputArgumentTypes = new[] { ExampleArgumentType.Integer },
        TrainingCount = 100,
        TestCount = 1000,
        EnabledDataTypes = DataTypes.Exec | DataTypes.Integer | DataTypes.Boolean | DataTypes.Char | DataTypes.String,
        MaxSize = 800,
        EvalLimit = 1600,
        ErcOptions = {
          ErcProbability = 0.05,
          StringErcOptions = new StringErcOptions(
            new StringConstantErcValue("The number of syllables is ", "aeiouy" ),
            new StringRandomErcValue {
              AllowLowercaseLetters = true,
              AllowUppercaseLetters = false,
              AllowDigits = true,
              AllowSymbols = true,
              AllowSpace = true,
              SpaceProbability = 0.2,
              VowelProbability = 0.2
            }),
          CharErcOptions = new CharErcOptions (
            new IntegerConstantErcValue('a', 'e', 'i', 'o', 'u'),
            new IntegerRangeErcValue(0x20, 0x7e))
        }
      };
    }

    protected override Example ParseExample(string[] input, string[] output) {
      return new Example {
        InputArgs = input,
        OutputArgs = output,
        InputString = input,
        OutputPrint = output[0],
      };
    }
  }
}
