﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  /// <summary>
  ///     Swaps the top two values.
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class SwapExpression<T> : StatelessExpression {
    public bool Eval(IPushStack<T> stack) {
      if (stack.Count < 2)
        return false;

      stack.Swap(2);
      return true;
    }
  }

  [PushExpression(StackTypes.Integer, "INTEGER.SWAP")]
  public class IntegerSwapExpression : SwapExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.Float, "FLOAT.SWAP")]
  public class FloatSwapExpression : SwapExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.Boolean, "BOOLEAN.SWAP")]
  public class BooleanSwapExpression : SwapExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Name, "NAME.SWAP")]
  public class NameSwapExpression : SwapExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.NameStack);
    }
  }

  [PushExpression(StackTypes.Exec, "EXEC.SWAP")]
  public class ExecSwapExpression : SwapExpression<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.ExecStack);
    }
  }

  [PushExpression(StackTypes.Code, "CODE.SWAP")]
  public class CodeSwapExpression : SwapExpression<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CodeStack);
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.SWAP")]
  public class CharSwapExpression : SwapExpression<char> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CharStack);
    }
  }

  [PushExpression(StackTypes.String, "STRING.SWAP")]
  public class StringSwapExpression : SwapExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringStack);
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].SWAP")]
  public class IntegerVectorSwapExpression : SwapExpression<List<long>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].SWAP")]
  public class FloatVectorSwapExpression : SwapExpression<List<double>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].SWAP")]
  public class BooleanVectorSwapExpression : SwapExpression<List<bool>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].SWAP")]
  public class StringVectorSwapExpression : SwapExpression<List<string>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack);
    }
  }
}