﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Takes the rest of the top item on the type stack.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorRestExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack) {
      if (vectorStack.IsEmpty || vectorStack.Top.Count < 2)
        return false;

      var result = vectorStack.Top.GetRange(1, vectorStack.Top.Count - 1);
      vectorStack.SetTop(result);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].REST")]
  public class IntegerVectorRestExpression : VectorRestExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].REST")]
  public class FloatVectorRestExpression : VectorRestExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].REST")]
  public class BooleanVectorRestExpression : VectorRestExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].REST")]
  public class StringVectorRestExpression : VectorRestExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack);
    }
  }
}
